/*
 In-line Network Cable Tracer (INCT)
 Version: 0.3
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 22/04/2013
 Last Modified: 16/07/2013
 
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define T16MS 0
#define T32MS 1
#define T64MS 2
#define T128MS 3
#define T250MS 4
#define T500MS 5
#define T1S 6
#define T2S 7
#define T4S 8
#define T8S 9

#define RX 0
#define TX 1

void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Watchdog timeout values
// 0=16ms, 1=32ms, 2=64ms, 3=128ms, 4=250ms, 5=500ms, 6=1sec, 7=2sec, 8=4sec, 9=8sec
// Used from http://interface.khm.de/index.php/lab/experiments/sleep_watchdog_battery/
void setup_watchdog(int ii) {
	byte bb;
	if (ii > 9 ) ii=9;
	bb=ii & 7;
	if (ii > 7) bb|= (1<<5);
	bb|= (1<<WDCE);
	
	MCUSR &= ~(1<<WDRF);
	// Start timed sequence
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	// Set new watchdog timeout value
	WDTCSR = bb;
	WDTCSR |= _BV(WDIE);
}

// Turn off the Watchdog
void turn_off_watchdog(void) {
	cli(); // Turn interrupts off
	MCUSR &= ~(1<<WDRF);
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	WDTCSR = 0;
	sei(); // Turn interrupts back on
}

// Watchdog sleep function - combines all the above functions into one
void watchdog_sleep(int timeout) { 
	setup_watchdog(timeout);
	system_sleep();
	turn_off_watchdog();
}

// ADC Read
int analog_read(uint8_t pin) {
	uint8_t low, high;
	
	// Set the analog reference
	ADMUX = pin & 0x3f;
	
	set_sleep_mode(SLEEP_MODE_ADC); // Set sleep mode ADC
	sleep_mode(); // System sleeps here
	
	// we have to read ADCL first; doing so locks both ADCL
	// and ADCH until ADCH is read.  reading ADCL second would
	// cause the results of each conversion to be discarded,
	// as ADCL and ADCH would be locked when it completed.
	low = ADCL;
	high = ADCH;
	
	// combine the two bytes
	return (high << 8) | low;
}

// Setup the ATtiny
void setup(void) {
	
	// Turn on ADC interrupt
	sbi(ADCSRA, ADIE);
	
	// Set outputs
	DDRB |= (1<<ledPin);
	DDRA |= (1<<txPlusControlPin);
	DDRA |= (1<<txMinusControlPin);
	DDRA |= (1<<rxPlusControlPin);
	DDRA |= (1<<rxMinusControlPin);
	
	// Set pull ups
	PORTB |= (1<<PB1);
	PORTB |= (1<<PB2);
	PORTA |= (1<<PA7);
	PORTA |= (1<<PA6);
	PORTA |= (1<<PA1);
	
	sei(); // Turn on interrupts
}